=== modified file 'lib/lp/registry/browser/tests/test_distroseries.py'
--- lib/lp/registry/browser/tests/test_distroseries.py	2011-07-06 14:42:11 +0000
+++ lib/lp/registry/browser/tests/test_distroseries.py	2011-07-06 16:19:18 +0000
@@ -28,9 +28,13 @@
 from zope.component import getUtility
 from zope.security.proxy import removeSecurityProxy
 
+from canonical.config import config
 from canonical.database.constants import UTC_NOW
 from canonical.database.sqlbase import flush_database_caches
-from canonical.launchpad.testing.pages import find_tag_by_id
+from canonical.launchpad.testing.pages import (
+    extract_text,
+    find_tag_by_id,
+    )
 from canonical.launchpad.webapp.authorization import check_permission
 from canonical.launchpad.webapp.batching import BatchNavigator
 from canonical.launchpad.webapp.interaction import get_current_principal
@@ -50,7 +54,6 @@
     RESOLVED,
     seriesToVocab,
     )
-from canonical.config import config
 from lp.registry.enum import (
     DistroSeriesDifferenceStatus,
     DistroSeriesDifferenceType,
@@ -93,6 +96,7 @@
     )
 from lp.testing.fakemethod import FakeMethod
 from lp.testing.matchers import (
+    DocTestMatches,
     EqualsIgnoringWhitespace,
     HasQueryCount,
     )
@@ -349,9 +353,6 @@
             soupmatchers.Tag(
                 'Derived series', 'h2',
                 text='Series initialization has failed'),
-            # soupmatchers.Tag(
-            #     'Init message', True,
-            #     text=re.compile('\s*This series is initializing.\s*')),
             )
         with person_logged_in(self.simple_user):
             view = create_initialized_view(
@@ -447,6 +448,72 @@
         self.assertInitSeriesLinkNotPresent(series, 'admin')
 
 
+class TestDistroSeriesDerivationPortlet(TestCaseWithFactory):
+
+    layer = LaunchpadFunctionalLayer
+
+    @property
+    def job_source(self):
+        return getUtility(IInitializeDistroSeriesJobSource)
+
+    def test_initialization_failed_can_retry(self):
+        # When initialization has failed and the user has the ability to retry
+        # it prompts the user to try again.
+        set_derived_series_ui_feature_flag(self)
+        series = self.factory.makeDistroSeries()
+        parent = self.factory.makeDistroSeries()
+        job = self.job_source.create(series, [parent.id])
+        job.start()
+        job.fail()
+        with person_logged_in(series.owner):
+            view = create_initialized_view(series, '+portlet-derivation')
+            html_content = view()
+        self.assertThat(
+            extract_text(html_content), DocTestMatches(
+                "Series initialization has failed\n"
+                "You can attempt initialization again."))
+
+    def test_initialization_failed_cannot_retry(self):
+        # When initialization has failed and the user does not have the
+        # ability to retry it suggests contacting someone who can.
+        set_derived_series_ui_feature_flag(self)
+        series = self.factory.makeDistroSeries()
+        parent = self.factory.makeDistroSeries()
+        job = self.job_source.create(series, [parent.id])
+        job.start()
+        job.fail()
+        with anonymous_logged_in():
+            view = create_initialized_view(series, '+portlet-derivation')
+            html_content = view()
+        self.assertThat(
+            extract_text(html_content), DocTestMatches(
+                "Series initialization has failed\n"
+                "You cannot attempt initialization again, "
+                "but Person-... may be able to help."))
+
+    def test_initialization_failed_cannot_retry_owner_is_team(self):
+        # When initialization has failed and the user does not have the
+        # ability to retry it suggests contacting someone who can. When the
+        # owner is a team the message differs slightly from when the owner is
+        # an individual.
+        set_derived_series_ui_feature_flag(self)
+        series = self.factory.makeDistroSeries()
+        with person_logged_in(series.distribution.owner):
+            series.distribution.owner = self.factory.makeTeam()
+        parent = self.factory.makeDistroSeries()
+        job = self.job_source.create(series, [parent.id])
+        job.start()
+        job.fail()
+        with anonymous_logged_in():
+            view = create_initialized_view(series, '+portlet-derivation')
+            html_content = view()
+        self.assertThat(
+            extract_text(html_content), DocTestMatches(
+                "Series initialization has failed\n"
+                "You cannot attempt initialization again, but "
+                "a member of Team ... may be able to help."))
+
+
 class TestMilestoneBatchNavigatorAttribute(TestCaseWithFactory):
     """Test the series.milestone_batch_navigator attribute."""
 

=== modified file 'lib/lp/registry/templates/distroseries-portlet-derivation.pt'
--- lib/lp/registry/templates/distroseries-portlet-derivation.pt	2011-07-06 14:31:44 +0000
+++ lib/lp/registry/templates/distroseries-portlet-derivation.pt	2011-07-06 16:19:18 +0000
@@ -69,5 +69,19 @@
       define="job context/getInitializationJob"
       condition="job/status/enumvalue:FAILED|nothing">
     <h2>Series initialization has failed</h2>
+    <tal:retry
+        define="can_retry context/@@+initseries/required:launchpad.Edit ">
+      <tal:can_retry condition="can_retry">
+        You can <a tal:attributes="href context/fmt:url/+initseries">
+        attempt initialization again</a>.
+      </tal:can_retry>
+      <tal:cannot_retry
+          define="owner context/owner"
+          condition="not:can_retry">
+        You cannot attempt initialization again, but <tal:team
+        condition="owner/is_team"> a member of </tal:team> <tal:owner
+        replace="structure owner/fmt:link" /> may be able to help.
+      </tal:cannot_retry>
+    </tal:retry>
   </tal:failed_initialization>
 </div>

